from django import template
from django.conf import settings
from django.templatetags.static import static
from django.urls import reverse

register = template.Library()


@register.simple_tag
def bundled_static(path):
    if settings.DEBUG:
        return reverse('django_static_bundler:bundle-static-file', args=(path,))

    return static(path)



import sys
from tornado.web import Application
from tornado.ioloop import IOLoop

from livesync.core.event import ClientEvent
from livesync.core.signals import livesync_event
from .handler import LiveSyncSocketHandler
from socket import error
import time


class LiveSyncSocketServer(Application):
    def __init__(self,port=9001):
        self.port = port
        super(LiveSyncSocketServer, self).__init__([(r"/", LiveSyncSocketHandler)])

    def server_close(self):
        ioloop = IOLoop.instance()
        ioloop.add_callback(ioloop.stop)
        ioloop.clear_current()
        IOLoop.clear_instance()

    def start(self, started_event=None):
        try:
            self.listen(self.port)
            if started_event:
                # inform the main thread the server can be started.
                started_event.set()
            instance = IOLoop.instance()
            if not instance.asyncio_loop.is_running():
                instance.start()
        except KeyboardInterrupt:
            self.server_close()
            sys.exit(0)
        except error as err:
            if err.errno == 98:
                time.sleep(1)
                livesync_event.send(sender=self.__class__, event=ClientEvent(
                    action='refresh',
                    parameters={}
                ))
                sys.exit(0)
            else:
                sys.exit(1)





class Webcomponent:
    def __init__(self):
        self.registry = {}

    def get(self, tag):
        return self.registry.get(tag, None)

    def define(self, tag, klass):
        self.registry[tag] = klass


webcomponent = Webcomponent()



import os
import itertools
from pathlib import Path

from django.conf import settings
from django.apps import apps
from django.template.utils import get_app_template_dirs

from livesync.core.event import ClientEvent
from livesync.core.signals import livesync_event
from livesync.fswatcher.handlers import BaseEventHandler


class LiveReloadRequestHandler(BaseEventHandler):
    @property
    def watched_paths(self):
        """
        list of paths to be watched for modifications on static files.

        Returns:
        all static and template locations for all apps included in DJANGO_LIVESYNC.INCLUDED_APPS.
        """
        tmpls = []
        for d in get_app_template_dirs(''):
            d = Path(d)
            if Path(settings.BASE_DIR) in d.parents and d.name in settings.DJANGO_LIVESYNC['INCLUDED_APPS'] and (d / 'templates').exists():
                tmpls.append(d / 'templates')

        paths = set(itertools.chain(
            getattr(settings, 'STATICFILES_DIRS', []),
            getattr(settings, 'TEMPLATE_DIRS', []), # django backward compatibility.
            tmpls,
        ))

        paths.update(*[tmpl['DIRS'] for tmpl in settings.TEMPLATES])

        if apps.is_installed('django.contrib.staticfiles'):
            from django.contrib.staticfiles.finders import get_finder
            finder = get_finder('django.contrib.staticfiles.finders.AppDirectoriesFinder')
            paths.update([
                st.location for app, st in finder.storages.items()
                if app in settings.DJANGO_LIVESYNC['INCLUDED_APPS']
            ])

        return paths

    def handle(self, event):
        livesync_event.send(sender=self.__class__, event=ClientEvent(
            action='refresh',
            parameters={}
        ))



from django.http import HttpResponse, HttpResponseRedirect
from django.template.loader import get_template


class Response:
    def __init__(self, request):
        self.request = request
        self.template_name = None
        self.context = None

    def from_template(self, template_name, context=None):
        self.template_name = template_name
        self.context = context or {}
        return self

    def to_200(self):
        data = get_template(self.template_name, using='samon').render(context=self.context, request=self.request)
        resp = HttpResponse(data)
        return resp

    def to_201(self):
        data = get_template(self.template_name, using='samon').render(context=self.context, request=self.request, to='js_template')
        resp = HttpResponse('m.html`' + data + '`')
        resp.status_code = 201
        return resp

    def to_403(self):
        data = get_template(self.template_name, using='samon').render(context=self.context, request=self.request, to='js_template')
        resp = HttpResponse('m.html`' + data + '`')
        resp.status_code = 403
        return resp

    def to_340(self, redirect_to):
        resp = HttpResponseRedirect(redirect_to=redirect_to)
        resp.status_code = 340
        return resp

    def to_ajax_redirect(self, redirect_to):
        self.to_340(redirect_to)


def response(request):
    return Response(request)



import mimetypes
import posixpath
from importlib.util import find_spec

import sass

from pathlib import Path

from django.conf import settings
from django.contrib.staticfiles.finders import find
from django.http import request, FileResponse, HttpResponse, Http404
from django.utils._os import safe_join
from django.views.static import serve

from django_brython.brython import make_package


def bundle_static_file(request, path):
    path = Path(posixpath.normpath(path).lstrip('/'))

    if path.suffixes[-2:] == ['.scss', '.css']:
        fullpath = find(path.parent / path.stem)
        result = sass.compile(filename=fullpath)

        response = HttpResponse(result, content_type='text/css')
        return response
    elif path.suffixes[-2:] == ['.brython', '.js']:
        module_path = '.'.join(str(path).split('/')[-1].split('.')[:-2])

        try:
            module = find_spec(module_path)
        except ModuleNotFoundError:
            raise Http404(f"'import {module_path}' exception, file: {path} can't be served")

        exclude_modules = getattr(settings, 'BRYTHON_BUNDLER_EXCLUDE', [])
        out = make_package.make(package_name=module.name,
                                package_path=module.submodule_search_locations[0],
                                exclude_modules=exclude_modules)

        return HttpResponse(out, content_type='text/javascript')
        # kirui/core/component.brython.js

    print(path)
    return serve(request, str(path), document_root=settings.STATIC_ROOT)



from pathlib import Path

from .exceptions import TemplateNotFound


class BaseLoader:
    def get_source(self, name):
        pass


class FileSystemLoader:
    def __init__(self, search_path):
        if isinstance(search_path, str):
            self.search_path = [Path(search_path).expanduser()]
        elif isinstance(search_path, Path):
            self.search_path = [search_path]
        else:
            self.search_path = [Path(_).expanduser() for _ in search_path]

    def get_source(self, name) -> [str, Path]:
        for path in self.search_path:
            if (source_path := path / name).is_file():
                with source_path.open('r', encoding='utf-8') as f:
                    return f.read(), source_path
        else:
            checked = ','.join(str(_) for _ in self.search_path)
            raise TemplateNotFound(f'{name} not found on path(s) {checked}')



import datetime
import time

from django.http import HttpResponse, JsonResponse, HttpResponseRedirect
from django.shortcuts import render
from django.core.cache import cache
from django.template.loader import get_template
from django.views import View

from django_kirui.http import response
from django_kirui.utils.paginator import Paginator
from kirui_devserver.backend.forms import SampleForm, FilterForm
from kirui_devserver.backend.models import Activity


def index(request):
    return response(request).from_template('xml/index.html').to_200()


class FormView(View):
    def get(self, request):
        form = SampleForm(request.POST or None, request.FILES or None)
        return response(request).from_template('xml/form.html', context={'form': form}).to_200()

    def post(self, request):
        # print(request.POST, request.FILES)
        form = SampleForm(request.POST or None, request.FILES or None, instance=Activity.objects.first())
        if form.is_valid():
            return response(request).to_340('/backend/index/')
        else:
            return response(request).from_template('xml/form_data.html', {'form': form}).to_403()


def modal(request):
    form = SampleForm(request.POST or None, request.FILES or None)
    form.now = datetime.datetime.now()
    if form.is_valid():
        return HttpResponse('OK')

    if form.is_valid():
        return response(request).from_template('xml/modal.html', context={'form': form}).to_201()
    else:
        return response(request).from_template('xml/modal.html', context={'form': form}).to_403()

    return resp


def table(request):
    data = []
    for row in range(1, 200):
        data.append({'first': f'{row}.1', 'second': f'{row}.2'})

    paginate_to = int(request.GET.get('paginate_to', 1))
    p = Paginator(data, 30)
    page = p.page(paginate_to)

    if request.GET.get('paginate_to', None):
        return response(request).from_template('xml/table_data.html', context={'page': page}).to_201()
    else:
        return response(request).from_template('xml/table.html', context={'page': page}).to_200()


class FilteredTable(View):
    def dispatch(self, request, *args, **kwargs):
        data = []
        for row in range(1, 20):
            data.append({'first': f'{row}.1', 'second': f'{row}.2'})
        self.data = data
        return super().dispatch(request, *args, **kwargs)

    def get(self, request):
        form = FilterForm()
        p = Paginator(self.data, 4)
        page = p.page(1)
        return response(request).from_template('backend/filtered_table.html', context={'form': form, 'page': page}).to_200()

    def post(self, request):
        form = FilterForm(request.POST)
        if form.is_valid():
            data = [row for row in self.data if row['first'].startswith(form.cleaned_data['field'])]
        p = Paginator(data, 4)

        paginate_to = int(request.GET.get('paginate_to', 1))
        page = p.page(paginate_to)
        return response(request).from_template('backend/filtered_table_data.html', context={'form': form, 'page': page}).to_201()


def charts(request):
    donut = {'first': 10, 'second': 20, 'third': 30}
    return response(request).from_template('backend/charts.html', context={'donut': donut}).to_200()


def card(request):
    return response(request).from_template('backend/card.html', context={}).to_200()
